﻿using System;
using System.Collections.Generic;
using System.Linq;
using Autofac;
using Autofac.Integration.Mef;

namespace LazyInAutofac2
{
    public interface ILogAppender
    {
        void Write(string text);
    }

    public interface ILogAppenderMetadata
    {
        string AppenderName { get; }
    }

    public class PrinterLogAppender : ILogAppender
    {
        public PrinterLogAppender()
        {
            Console.WriteLine("PrinterLogAppender created.");
        }

        public void Write(string text)
        {
            Console.WriteLine("[Printer] " + text);
        }
    }

    public class ScreenLogAppender : ILogAppender
    {
        public ScreenLogAppender()
        {
            Console.WriteLine("ScreenLogAppender created.");
        }

        public void Write(string text)
        {
            Console.WriteLine("[Screen] " + text);
        }
    }

    public class Log
    {
        readonly IEnumerable<Lazy<ILogAppender, ILogAppenderMetadata>> _appenders;

        public Log(IEnumerable<Lazy<ILogAppender, ILogAppenderMetadata>> appenders)
        {
            _appenders = appenders;
        }

        public void Write(string destination, string message)
        {
            var appender = _appenders.First(a => a.Metadata.AppenderName == destination);
            appender.Value.Write(message);
        }
    }

    public class Program
    {
        readonly Log _log;

        public Program(Log log)
        {
            _log = log;
        }

        public void Run()
        {
            _log.Write("printer", "Breeeeeep...");
            _log.Write("screen", "Zip!");
            _log.Write("printer", "Clat tac lac clak.");
            Console.ReadKey(true);
        }

        static void Main(string[] args)
        {
            var builder = new ContainerBuilder();

            builder.RegisterModule(new LazyDependencyModule());

            builder.RegisterType<Program>();
            builder.RegisterType<Log>();

            builder.Register(c => new PrinterLogAppender())
                .As<ILogAppender>()
                .WithExtendedProperty("AppenderName", "printer");

            builder.Register(c => new ScreenLogAppender())
                .As<ILogAppender>()
                .WithExtendedProperty("AppenderName", "screen");

            using (var container = builder.Build())
            {
                container.Resolve<Program>().Run();
            }
        }
    }
}
